# Generated by roxygen2: do not edit by hand

export(calculate_area_intersection_weights)
export(create_cell_geometry)
export(read_attribute_data)
export(read_geometry)
export(read_timeseries_dsg)
export(write_attribute_data)
export(write_geometry)
export(write_timeseries_dsg)
importFrom(RNetCDF,att.put.nc)
importFrom(RNetCDF,close.nc)
importFrom(RNetCDF,create.nc)
importFrom(RNetCDF,dim.def.nc)
importFrom(RNetCDF,dim.inq.nc)
importFrom(RNetCDF,open.nc)
importFrom(RNetCDF,utcal.nc)
importFrom(RNetCDF,var.def.nc)
importFrom(RNetCDF,var.get.nc)
importFrom(RNetCDF,var.inq.nc)
importFrom(RNetCDF,var.put.nc)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,ungroup)
importFrom(methods,is)
importFrom(ncmeta,nc_atts)
importFrom(ncmeta,nc_axes)
importFrom(ncmeta,nc_dim)
importFrom(ncmeta,nc_meta)
importFrom(sf,"st_crs<-")
importFrom(sf,st_area)
importFrom(sf,st_as_sf)
importFrom(sf,st_bbox)
importFrom(sf,st_buffer)
importFrom(sf,st_contains)
importFrom(sf,st_convex_hull)
importFrom(sf,st_coordinates)
importFrom(sf,st_crs)
importFrom(sf,st_drop_geometry)
importFrom(sf,st_geometry)
importFrom(sf,st_geometry_type)
importFrom(sf,st_intersection)
importFrom(sf,st_join)
importFrom(sf,st_linestring)
importFrom(sf,st_multilinestring)
importFrom(sf,st_multipolygon)
importFrom(sf,st_polygon)
importFrom(sf,st_set_geometry)
importFrom(sf,st_sf)
importFrom(sf,st_sfc)
importFrom(sf,st_transform)
importFrom(sf,st_union)
importFrom(sf,st_zm)
importFrom(stars,st_as_stars)
importFrom(stars,st_dimensions)
importFrom(stats,setNames)
